#include <stdio.h>
#include <stdlib.h>
#include <vga.h>
#include <vgakeyboard.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include "gui.h"
#include "loopdata.h"
#include "input.h"
#include "fxdata.h"
#include "loadlayout.h"

// structure of layout file

typedef struct _fhead
{
	int ID;
	int isFX;
	int isLoops;
	// must save, useful for both FX and Loops
	int mouse_spinner[8];
	int mouse_axis[8];
}	fhead;

typedef struct _fxdata
{
	effect_stat fx[NUM_LOOPS*FX_ROW];
	loopbus_stat blinks[NUM_LOOPS];
}	fxdata;

typedef struct _loopparms
{
	int BPM;
	int parms[6*NUM_LOOPS];
	int parmlink[6*NUM_LOOPS];
	int measures[NUM_LOOPS];
	int record[NUM_LOOPS];
	int fx[NUM_LOOPS];
	int looplen[NUM_LOOPS];
}	loopparms;



char load_error[512];

char * load_layout(char * filename)
{
	fhead l_fhead;
	fxdata l_fxdata;
	loopparms l_loopparms;
	int i, loop, col, index;
	FILE * inFile;
	strcpy(load_error, "load ok");
	// open file
	inFile = fopen(filename, "rb");
	if (inFile == NULL)
	{
		strcpy (load_error, "error opening file to load");
		return load_error;
	}
	// read in head structure
	if (fread(&l_fhead, sizeof(l_fhead), 1, inFile) != 1)
	{
		strcpy (load_error, "error #1 reading file");
		fclose (inFile);
		return load_error;
	}
	if (l_fhead.ID != 'ele2')
	{
		strcpy (load_error, "not an ELE file!");
		fclose (inFile);
		return load_error;
	}
	if (l_fhead.isFX)
		if (fread(&l_fxdata, sizeof(l_fxdata), 1, inFile) != 1)
		{
		strcpy (load_error, "error #2 reading file");
		fclose (inFile);
		return load_error;
		}
	
	if (l_fhead.isLoops)
		if (fread(&l_loopparms, sizeof(l_loopparms), 1, inFile) != 1)
		{
		strcpy (load_error, "error #3 reading file");
		fclose (inFile);
		return load_error;
		}
	
	// set mouse spinners
	for (i=0;i<8;i++)
	{
		mouse_spinner_value[i] = l_fhead.mouse_spinner[i];
		mouse_spinner_axis[i]  = l_fhead.mouse_axis[i];
	}
	
	// do effects settings - be careful!
	// get the mutex
	fx_lock();
	if (l_fhead.isFX)
	{
		index = 0;
		for (loop = 0; loop < NUM_LOOPS; loop++)
		{
			for (col = 0; col < FX_ROW; col++)
			{
				// clear the extra data area
				if (fx_layout[loop][col].extra != NULL)
					free(fx_layout[loop][col].extra);
				fx_layout[loop][col].extra = NULL;
				// now copy the new stuff in.
				fx_layout[loop][col].type = l_fxdata.fx[index].type;
				for (i=0;i<4;i++)
				{
				fx_layout[loop][col].parmlink[i]=l_fxdata.fx[index].parmlink[i];
				fx_layout[loop][col].parms[i]   =l_fxdata.fx[index].parms[i];
				}
				for (i=0;i<2;i++)
				{
				fx_layout[loop][col].link_is[i] =l_fxdata.fx[index].link_is[i];
				fx_layout[loop][col].link_Col[i] = l_fxdata.fx[index].link_Col[i];
				fx_layout[loop][col].link_Term[i] = l_fxdata.fx[index].link_Term[i];
				fx_layout[loop][col].link_Loop[i] = l_fxdata.fx[index].link_Loop[i];
				
				}
				index++;
			}
			loopbus_links[loop] = l_fxdata.blinks[loop] ;
			
			
			
		}
		// just to avoid problems with parameter pane pointing to
		// altered or nonexistent effect ........................
		clear_curr_effect_scr3();
	}
	fx_unlock();
	index = 0;
	// do loop parameters
	if (l_fhead.isLoops)
		for (loop = 0; loop < NUM_LOOPS; loop++)
	{
		for (col = 0; col < 6; col++)
		{
			scr1_parmlink[col][loop] = l_loopparms.parmlink[index];
			scr1_parms[col][loop] = l_loopparms.parms[index];
			index++;
		}
		scr1_measures[loop] = l_loopparms.measures[loop];
		scr1_fx[loop] = l_loopparms.fx[loop];	
	}
	if (l_fhead.isLoops) set_bpm(l_loopparms.BPM);
	
	// load loops from file
	if (l_fhead.isLoops)
		for (loop = 0; loop < NUM_LOOPS; loop++)
	{
		// keep the audio thread from touching this //
		lockLoop(loop);
		deleteLoop(loop); // clear it to be sure
		if (l_loopparms.looplen[loop] != 0)
		{
			// now get ready to load the damn thing!
			loopData[loop] = malloc (l_loopparms.looplen[loop]);
			if (fread(loopData[loop], l_loopparms.looplen[loop], 1, inFile) != 1)
			{
				strcpy (load_error, "error #4 reading file");
				fclose (inFile);
				free(loopData[loop]);
				loopData[loop] = NULL;
				return load_error;
			}
			loopLength[loop] = l_loopparms.looplen[loop];
		}
		unlockLoop(loop);
	}
	
	fclose (inFile);
	return load_error;
}

char * save_layout(char * filename, int saveLoops, int saveFX)
{
	fhead s_fhead;
	fxdata s_fxdata;
	loopparms s_loopparms;
	int i, loop, col, index;
	FILE * outFile;
	strcpy(load_error, "save ok");
	
	// fill in head
	s_fhead.ID = 'ele2';
	s_fhead.isFX = saveFX;
	s_fhead.isLoops = saveLoops;
	for (i=0;i<8;i++)
	{
		s_fhead.mouse_spinner[i] = mouse_spinner_value[i];
		s_fhead.mouse_axis[i] = mouse_spinner_axis[i];
	}
	
	// fill in fxdata
	index = 0;
	for (loop = 0; loop < NUM_LOOPS; loop++)
	{
		for (col = 0; col < FX_ROW; col++)
		{
			s_fxdata.fx[index] = fx_layout[loop][col];
			index++;
		}
		s_fxdata.blinks[loop] = loopbus_links[loop];
	}
	
	// fill in loopparms
	index = 0;
	s_loopparms.BPM = get_bpm();
	for (loop = 0; loop < NUM_LOOPS; loop++)
	{
		for (col = 0; col < 6; col++)
		{
			s_loopparms.parms[index] = scr1_parms[col][loop];
			s_loopparms.parmlink[index] = scr1_parmlink[col][loop];
			index++;
		}
		s_loopparms.measures[loop] = scr1_measures[loop];
		s_loopparms.fx[loop] = scr1_fx[loop];
		s_loopparms.looplen[loop] = loopLength[loop];
	}
	// save our beautiful structs!
	outFile = fopen (filename, "wb");
	if (outFile == NULL)
	{
		strcpy (load_error, "error opening file to save");
		return load_error;
	}
	
	if (fwrite(&s_fhead, sizeof(s_fhead), 1, outFile) != 1 )
	{
		strcpy (load_error, "error #1 writing file");
		fclose(outFile);
		return load_error;
	}
	
	if (saveFX)
		if (fwrite(&s_fxdata, sizeof(s_fxdata), 1, outFile) != 1 )
	{
		strcpy (load_error, "error #2 writing file");
		fclose(outFile);
		return load_error;
	}
	
	if (saveLoops)
		if (fwrite(&s_loopparms, sizeof(s_loopparms), 1, outFile) != 1 )
	{
		strcpy (load_error, "error #3 writing file");
		fclose(outFile);
		return load_error;
	}
	
	// save the raw loop audio
	if (saveLoops)
	{
		for (loop = 0; loop < NUM_LOOPS; loop++)
			if (fwrite(loopData[loop], loopLength[loop], 1, outFile) != 1)
			{
			strcpy (load_error, "error #4 writing file");
			fclose(outFile);
			return load_error;
			}
			
		
		
	}
	
	fclose(outFile);
	return load_error;
}
